# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "SubscriptionListResponse",
    "Addon",
    "Budget",
    "Coupon",
    "CouponAmountsOff",
    "FutureUpdate",
    "FutureUpdateTargetPackage",
    "LatestInvoice",
    "MinimumSpend",
    "Price",
    "PriceTier",
    "PriceTierFlatPrice",
    "PriceTierUnitPrice",
    "SubscriptionEntitlement",
    "Trial",
]


class Addon(BaseModel):
    """Addon configuration"""

    id: str
    """Addon ID"""

    quantity: int
    """Number of addon instances"""


class Budget(BaseModel):
    """Budget configuration"""

    has_soft_limit: bool = FieldInfo(alias="hasSoftLimit")
    """Whether the budget is a soft limit"""

    limit: float
    """Maximum spending limit"""


class CouponAmountsOff(BaseModel):
    amount: Optional[float] = None
    """The price amount"""

    currency: Optional[
        Literal[
            "usd",
            "aed",
            "all",
            "amd",
            "ang",
            "aud",
            "awg",
            "azn",
            "bam",
            "bbd",
            "bdt",
            "bgn",
            "bif",
            "bmd",
            "bnd",
            "bsd",
            "bwp",
            "byn",
            "bzd",
            "brl",
            "cad",
            "cdf",
            "chf",
            "cny",
            "czk",
            "dkk",
            "dop",
            "dzd",
            "egp",
            "etb",
            "eur",
            "fjd",
            "gbp",
            "gel",
            "gip",
            "gmd",
            "gyd",
            "hkd",
            "hrk",
            "htg",
            "idr",
            "ils",
            "inr",
            "isk",
            "jmd",
            "jpy",
            "kes",
            "kgs",
            "khr",
            "kmf",
            "krw",
            "kyd",
            "kzt",
            "lbp",
            "lkr",
            "lrd",
            "lsl",
            "mad",
            "mdl",
            "mga",
            "mkd",
            "mmk",
            "mnt",
            "mop",
            "mro",
            "mvr",
            "mwk",
            "mxn",
            "myr",
            "mzn",
            "nad",
            "ngn",
            "nok",
            "npr",
            "nzd",
            "pgk",
            "php",
            "pkr",
            "pln",
            "qar",
            "ron",
            "rsd",
            "rub",
            "rwf",
            "sar",
            "sbd",
            "scr",
            "sek",
            "sgd",
            "sle",
            "sll",
            "sos",
            "szl",
            "thb",
            "tjs",
            "top",
            "try",
            "ttd",
            "tzs",
            "uah",
            "uzs",
            "vnd",
            "vuv",
            "wst",
            "xaf",
            "xcd",
            "yer",
            "zar",
            "zmw",
            "clp",
            "djf",
            "gnf",
            "ugx",
            "pyg",
            "xof",
            "xpf",
        ]
    ] = None
    """The price currency"""


class Coupon(BaseModel):
    """Coupon applied to a subscription"""

    id: str
    """Coupon ID"""

    name: str
    """Coupon name"""

    status: Literal["ACTIVE", "EXPIRED", "REMOVED"]
    """Coupon status"""

    amounts_off: Optional[List[CouponAmountsOff]] = FieldInfo(alias="amountsOff", default=None)
    """Fixed amount discounts by currency"""

    percent_off: Optional[float] = FieldInfo(alias="percentOff", default=None)
    """Percentage discount"""


class FutureUpdateTargetPackage(BaseModel):
    """Target package for the update"""

    id: str
    """Target package for the update"""


class FutureUpdate(BaseModel):
    """Scheduled subscription update"""

    scheduled_execution_time: datetime = FieldInfo(alias="scheduledExecutionTime")
    """Scheduled execution time"""

    schedule_status: Literal["PENDING_PAYMENT", "SCHEDULED", "CANCELED", "DONE", "FAILED"] = FieldInfo(
        alias="scheduleStatus"
    )
    """Status of the scheduled update"""

    subscription_schedule_type: Literal[
        "DOWNGRADE",
        "PLAN",
        "BILLING_PERIOD",
        "UNIT_AMOUNT",
        "RECURRING_CREDITS",
        "PRICE_OVERRIDE",
        "ADDON",
        "COUPON",
        "MIGRATE_TO_LATEST",
        "ADDITIONAL_META_DATA",
        "BILLING_INFO_METADATA",
    ] = FieldInfo(alias="subscriptionScheduleType")
    """Type of scheduled change"""

    target_package: Optional[FutureUpdateTargetPackage] = FieldInfo(alias="targetPackage", default=None)
    """Target package for the update"""


class LatestInvoice(BaseModel):
    """Latest invoice for the subscription"""

    billing_id: str = FieldInfo(alias="billingId")
    """Invoice billing ID"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Invoice creation date"""

    requires_action: bool = FieldInfo(alias="requiresAction")
    """Whether payment requires action"""

    status: Literal["OPEN", "CANCELED", "PAID"]
    """Invoice status"""

    amount_due: Optional[float] = FieldInfo(alias="amountDue", default=None)
    """Amount due"""

    billing_reason: Optional[
        Literal[
            "BILLING_CYCLE",
            "SUBSCRIPTION_CREATION",
            "SUBSCRIPTION_UPDATE",
            "MANUAL",
            "MINIMUM_INVOICE_AMOUNT_EXCEEDED",
            "OTHER",
        ]
    ] = FieldInfo(alias="billingReason", default=None)
    """Billing reason"""

    currency: Optional[str] = None
    """Invoice currency"""

    pdf_url: Optional[str] = FieldInfo(alias="pdfUrl", default=None)
    """Invoice PDF URL"""

    total: Optional[float] = None
    """Total amount"""


class MinimumSpend(BaseModel):
    """Minimum spend configuration"""

    amount: Optional[float] = None
    """The price amount"""

    currency: Optional[
        Literal[
            "usd",
            "aed",
            "all",
            "amd",
            "ang",
            "aud",
            "awg",
            "azn",
            "bam",
            "bbd",
            "bdt",
            "bgn",
            "bif",
            "bmd",
            "bnd",
            "bsd",
            "bwp",
            "byn",
            "bzd",
            "brl",
            "cad",
            "cdf",
            "chf",
            "cny",
            "czk",
            "dkk",
            "dop",
            "dzd",
            "egp",
            "etb",
            "eur",
            "fjd",
            "gbp",
            "gel",
            "gip",
            "gmd",
            "gyd",
            "hkd",
            "hrk",
            "htg",
            "idr",
            "ils",
            "inr",
            "isk",
            "jmd",
            "jpy",
            "kes",
            "kgs",
            "khr",
            "kmf",
            "krw",
            "kyd",
            "kzt",
            "lbp",
            "lkr",
            "lrd",
            "lsl",
            "mad",
            "mdl",
            "mga",
            "mkd",
            "mmk",
            "mnt",
            "mop",
            "mro",
            "mvr",
            "mwk",
            "mxn",
            "myr",
            "mzn",
            "nad",
            "ngn",
            "nok",
            "npr",
            "nzd",
            "pgk",
            "php",
            "pkr",
            "pln",
            "qar",
            "ron",
            "rsd",
            "rub",
            "rwf",
            "sar",
            "sbd",
            "scr",
            "sek",
            "sgd",
            "sle",
            "sll",
            "sos",
            "szl",
            "thb",
            "tjs",
            "top",
            "try",
            "ttd",
            "tzs",
            "uah",
            "uzs",
            "vnd",
            "vuv",
            "wst",
            "xaf",
            "xcd",
            "yer",
            "zar",
            "zmw",
            "clp",
            "djf",
            "gnf",
            "ugx",
            "pyg",
            "xof",
            "xpf",
        ]
    ] = None
    """The price currency"""


class PriceTierFlatPrice(BaseModel):
    """The flat fee price of the price tier"""

    amount: float
    """The price amount"""

    currency: Literal[
        "usd",
        "aed",
        "all",
        "amd",
        "ang",
        "aud",
        "awg",
        "azn",
        "bam",
        "bbd",
        "bdt",
        "bgn",
        "bif",
        "bmd",
        "bnd",
        "bsd",
        "bwp",
        "byn",
        "bzd",
        "brl",
        "cad",
        "cdf",
        "chf",
        "cny",
        "czk",
        "dkk",
        "dop",
        "dzd",
        "egp",
        "etb",
        "eur",
        "fjd",
        "gbp",
        "gel",
        "gip",
        "gmd",
        "gyd",
        "hkd",
        "hrk",
        "htg",
        "idr",
        "ils",
        "inr",
        "isk",
        "jmd",
        "jpy",
        "kes",
        "kgs",
        "khr",
        "kmf",
        "krw",
        "kyd",
        "kzt",
        "lbp",
        "lkr",
        "lrd",
        "lsl",
        "mad",
        "mdl",
        "mga",
        "mkd",
        "mmk",
        "mnt",
        "mop",
        "mro",
        "mvr",
        "mwk",
        "mxn",
        "myr",
        "mzn",
        "nad",
        "ngn",
        "nok",
        "npr",
        "nzd",
        "pgk",
        "php",
        "pkr",
        "pln",
        "qar",
        "ron",
        "rsd",
        "rub",
        "rwf",
        "sar",
        "sbd",
        "scr",
        "sek",
        "sgd",
        "sle",
        "sll",
        "sos",
        "szl",
        "thb",
        "tjs",
        "top",
        "try",
        "ttd",
        "tzs",
        "uah",
        "uzs",
        "vnd",
        "vuv",
        "wst",
        "xaf",
        "xcd",
        "yer",
        "zar",
        "zmw",
        "clp",
        "djf",
        "gnf",
        "ugx",
        "pyg",
        "xof",
        "xpf",
    ]
    """The price currency"""


class PriceTierUnitPrice(BaseModel):
    """The unit price of the price tier"""

    amount: float
    """The price amount"""

    currency: Literal[
        "usd",
        "aed",
        "all",
        "amd",
        "ang",
        "aud",
        "awg",
        "azn",
        "bam",
        "bbd",
        "bdt",
        "bgn",
        "bif",
        "bmd",
        "bnd",
        "bsd",
        "bwp",
        "byn",
        "bzd",
        "brl",
        "cad",
        "cdf",
        "chf",
        "cny",
        "czk",
        "dkk",
        "dop",
        "dzd",
        "egp",
        "etb",
        "eur",
        "fjd",
        "gbp",
        "gel",
        "gip",
        "gmd",
        "gyd",
        "hkd",
        "hrk",
        "htg",
        "idr",
        "ils",
        "inr",
        "isk",
        "jmd",
        "jpy",
        "kes",
        "kgs",
        "khr",
        "kmf",
        "krw",
        "kyd",
        "kzt",
        "lbp",
        "lkr",
        "lrd",
        "lsl",
        "mad",
        "mdl",
        "mga",
        "mkd",
        "mmk",
        "mnt",
        "mop",
        "mro",
        "mvr",
        "mwk",
        "mxn",
        "myr",
        "mzn",
        "nad",
        "ngn",
        "nok",
        "npr",
        "nzd",
        "pgk",
        "php",
        "pkr",
        "pln",
        "qar",
        "ron",
        "rsd",
        "rub",
        "rwf",
        "sar",
        "sbd",
        "scr",
        "sek",
        "sgd",
        "sle",
        "sll",
        "sos",
        "szl",
        "thb",
        "tjs",
        "top",
        "try",
        "ttd",
        "tzs",
        "uah",
        "uzs",
        "vnd",
        "vuv",
        "wst",
        "xaf",
        "xcd",
        "yer",
        "zar",
        "zmw",
        "clp",
        "djf",
        "gnf",
        "ugx",
        "pyg",
        "xof",
        "xpf",
    ]
    """The price currency"""


class PriceTier(BaseModel):
    flat_price: Optional[PriceTierFlatPrice] = FieldInfo(alias="flatPrice", default=None)
    """The flat fee price of the price tier"""

    unit_price: Optional[PriceTierUnitPrice] = FieldInfo(alias="unitPrice", default=None)
    """The unit price of the price tier"""

    up_to: Optional[float] = FieldInfo(alias="upTo", default=None)
    """The up to quantity of the price tier"""


class Price(BaseModel):
    addon_id: Optional[str] = FieldInfo(alias="addonId", default=None)
    """Addon identifier for the price override"""

    amount: Optional[float] = None
    """The price amount"""

    base_charge: Optional[bool] = FieldInfo(alias="baseCharge", default=None)
    """Whether this is a base charge override"""

    billing_country_code: Optional[str] = FieldInfo(alias="billingCountryCode", default=None)
    """The billing country code of the price"""

    block_size: Optional[float] = FieldInfo(alias="blockSize", default=None)
    """Block size for pricing"""

    currency: Optional[
        Literal[
            "usd",
            "aed",
            "all",
            "amd",
            "ang",
            "aud",
            "awg",
            "azn",
            "bam",
            "bbd",
            "bdt",
            "bgn",
            "bif",
            "bmd",
            "bnd",
            "bsd",
            "bwp",
            "byn",
            "bzd",
            "brl",
            "cad",
            "cdf",
            "chf",
            "cny",
            "czk",
            "dkk",
            "dop",
            "dzd",
            "egp",
            "etb",
            "eur",
            "fjd",
            "gbp",
            "gel",
            "gip",
            "gmd",
            "gyd",
            "hkd",
            "hrk",
            "htg",
            "idr",
            "ils",
            "inr",
            "isk",
            "jmd",
            "jpy",
            "kes",
            "kgs",
            "khr",
            "kmf",
            "krw",
            "kyd",
            "kzt",
            "lbp",
            "lkr",
            "lrd",
            "lsl",
            "mad",
            "mdl",
            "mga",
            "mkd",
            "mmk",
            "mnt",
            "mop",
            "mro",
            "mvr",
            "mwk",
            "mxn",
            "myr",
            "mzn",
            "nad",
            "ngn",
            "nok",
            "npr",
            "nzd",
            "pgk",
            "php",
            "pkr",
            "pln",
            "qar",
            "ron",
            "rsd",
            "rub",
            "rwf",
            "sar",
            "sbd",
            "scr",
            "sek",
            "sgd",
            "sle",
            "sll",
            "sos",
            "szl",
            "thb",
            "tjs",
            "top",
            "try",
            "ttd",
            "tzs",
            "uah",
            "uzs",
            "vnd",
            "vuv",
            "wst",
            "xaf",
            "xcd",
            "yer",
            "zar",
            "zmw",
            "clp",
            "djf",
            "gnf",
            "ugx",
            "pyg",
            "xof",
            "xpf",
        ]
    ] = None
    """The price currency"""

    feature_id: Optional[str] = FieldInfo(alias="featureId", default=None)
    """Feature identifier for the price override"""

    tiers: Optional[List[PriceTier]] = None
    """Pricing tiers configuration"""


class SubscriptionEntitlement(BaseModel):
    """Subscription entitlement reference"""

    id: str
    """Feature ID or currency ID"""

    type: Literal["FEATURE", "CREDIT"]
    """Entitlement type (FEATURE or CREDIT)"""


class Trial(BaseModel):
    """Trial configuration"""

    trial_end_behavior: Literal["CONVERT_TO_PAID", "CANCEL_SUBSCRIPTION"] = FieldInfo(alias="trialEndBehavior")
    """Behavior when the trial ends"""


class SubscriptionListResponse(BaseModel):
    """Customer subscription to a plan"""

    id: str
    """Subscription ID"""

    billing_id: Optional[str] = FieldInfo(alias="billingId", default=None)
    """Billing ID"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Created at"""

    customer_id: str = FieldInfo(alias="customerId")
    """Customer ID"""

    payment_collection: Literal["NOT_REQUIRED", "PROCESSING", "FAILED", "ACTION_REQUIRED"] = FieldInfo(
        alias="paymentCollection"
    )
    """Payment collection"""

    plan_id: str = FieldInfo(alias="planId")
    """Plan ID"""

    pricing_type: Literal["FREE", "PAID", "CUSTOM"] = FieldInfo(alias="pricingType")
    """Pricing type"""

    start_date: datetime = FieldInfo(alias="startDate")
    """Subscription start date"""

    status: Literal["PAYMENT_PENDING", "ACTIVE", "EXPIRED", "IN_TRIAL", "CANCELED", "NOT_STARTED"]
    """Subscription status"""

    addons: Optional[List[Addon]] = None

    billing_cycle_anchor: Optional[datetime] = FieldInfo(alias="billingCycleAnchor", default=None)
    """Billing cycle anchor date"""

    budget: Optional[Budget] = None
    """Budget configuration"""

    cancellation_date: Optional[datetime] = FieldInfo(alias="cancellationDate", default=None)
    """Subscription cancellation date"""

    cancel_reason: Optional[
        Literal[
            "UPGRADE_OR_DOWNGRADE",
            "CANCELLED_BY_BILLING",
            "EXPIRED",
            "DETACH_BILLING",
            "TRIAL_ENDED",
            "Immediate",
            "TRIAL_CONVERTED",
            "PENDING_PAYMENT_EXPIRED",
            "ScheduledCancellation",
            "CustomerArchived",
            "AutoCancellationRule",
        ]
    ] = FieldInfo(alias="cancelReason", default=None)
    """Subscription cancel reason"""

    coupons: Optional[List[Coupon]] = None
    """Coupons applied to the subscription"""

    current_billing_period_end: Optional[datetime] = FieldInfo(alias="currentBillingPeriodEnd", default=None)
    """End of the current billing period"""

    current_billing_period_start: Optional[datetime] = FieldInfo(alias="currentBillingPeriodStart", default=None)
    """Start of the current billing period"""

    effective_end_date: Optional[datetime] = FieldInfo(alias="effectiveEndDate", default=None)
    """Subscription effective end date"""

    end_date: Optional[datetime] = FieldInfo(alias="endDate", default=None)
    """Subscription end date"""

    future_updates: Optional[List[FutureUpdate]] = FieldInfo(alias="futureUpdates", default=None)
    """Scheduled future updates for the subscription"""

    latest_invoice: Optional[LatestInvoice] = FieldInfo(alias="latestInvoice", default=None)
    """Latest invoice for the subscription"""

    metadata: Optional[Dict[str, str]] = None
    """Additional metadata for the subscription"""

    minimum_spend: Optional[MinimumSpend] = FieldInfo(alias="minimumSpend", default=None)
    """Minimum spend configuration"""

    paying_customer_id: Optional[str] = FieldInfo(alias="payingCustomerId", default=None)
    """Paying customer ID for delegated billing"""

    payment_collection_method: Optional[Literal["CHARGE", "INVOICE", "NONE"]] = FieldInfo(
        alias="paymentCollectionMethod", default=None
    )
    """The method used to collect payments for a subscription"""

    prices: Optional[List[Price]] = None

    resource_id: Optional[str] = FieldInfo(alias="resourceId", default=None)
    """Resource ID"""

    subscription_entitlements: Optional[List[SubscriptionEntitlement]] = FieldInfo(
        alias="subscriptionEntitlements", default=None
    )
    """Entitlements associated with the subscription"""

    trial: Optional[Trial] = None
    """Trial configuration"""

    trial_end_date: Optional[datetime] = FieldInfo(alias="trialEndDate", default=None)
    """Subscription trial end date"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ....._models import BaseModel

__all__ = ["CreditGrantResponse", "Data", "DataCost", "DataLatestInvoice"]


class DataCost(BaseModel):
    """The monetary cost of the credit grant"""

    amount: float
    """The cost amount"""

    currency: str
    """The currency code"""


class DataLatestInvoice(BaseModel):
    """The latest invoice details for this grant"""

    billing_id: str = FieldInfo(alias="billingId")
    """The billing provider invoice ID"""

    billing_reason: Optional[Literal["MANUAL", "OTHER"]] = FieldInfo(alias="billingReason", default=None)
    """The billing reason for the invoice"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """The invoice creation date"""

    currency: Optional[str] = None
    """The invoice currency"""

    due_date: Optional[datetime] = FieldInfo(alias="dueDate", default=None)
    """The invoice due date"""

    error_message: Optional[str] = FieldInfo(alias="errorMessage", default=None)
    """Error message if payment failed"""

    payment_url: Optional[str] = FieldInfo(alias="paymentUrl", default=None)
    """The payment URL for settling the invoice"""

    pdf_url: Optional[str] = FieldInfo(alias="pdfUrl", default=None)
    """The PDF URL of the invoice"""

    requires_action: bool = FieldInfo(alias="requiresAction")
    """Whether the invoice requires user action"""

    status: Literal["OPEN", "PAID", "CANCELED"]
    """The invoice status"""

    sub_total: Optional[float] = FieldInfo(alias="subTotal", default=None)
    """The subtotal amount before tax"""

    tax: Optional[float] = None
    """The tax amount"""

    total: Optional[float] = None
    """The total amount including tax"""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """The invoice last update date"""


class Data(BaseModel):
    """Credit grant object representing allocated credits for a customer"""

    id: str
    """The unique readable identifier of the credit grant"""

    amount: float
    """The total credits granted"""

    comment: Optional[str] = None
    """An optional comment on the credit grant"""

    consumed_amount: float = FieldInfo(alias="consumedAmount")
    """The total credits consumed from this grant"""

    cost: DataCost
    """The monetary cost of the credit grant"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Timestamp of when the record was created"""

    currency_id: str = FieldInfo(alias="currencyId")
    """The currency identifier for this grant"""

    customer_id: str = FieldInfo(alias="customerId")
    """The customer ID this grant belongs to"""

    display_name: str = FieldInfo(alias="displayName")
    """The display name of the credit grant"""

    effective_at: datetime = FieldInfo(alias="effectiveAt")
    """The date when the credit grant becomes effective"""

    expire_at: Optional[datetime] = FieldInfo(alias="expireAt", default=None)
    """The date when the credit grant expires"""

    grant_type: Literal["PAID", "PROMOTIONAL", "RECURRING"] = FieldInfo(alias="grantType")
    """The type of credit grant (PAID, PROMOTIONAL, RECURRING)"""

    invoice_id: Optional[str] = FieldInfo(alias="invoiceId", default=None)
    """The billing invoice ID associated with this grant"""

    latest_invoice: Optional[DataLatestInvoice] = FieldInfo(alias="latestInvoice", default=None)
    """The latest invoice details for this grant"""

    metadata: Dict[str, str]
    """Metadata associated with the entity"""

    payment_collection: Literal["NOT_REQUIRED", "PROCESSING", "FAILED", "ACTION_REQUIRED"] = FieldInfo(
        alias="paymentCollection"
    )
    """The payment collection status"""

    priority: float
    """The priority of the credit grant (lower number = higher priority)"""

    resource_id: Optional[str] = FieldInfo(alias="resourceId", default=None)
    """The resource ID this grant is scoped to"""

    source_type: Optional[Literal["PRICE", "PLAN_ENTITLEMENT", "ADDON_ENTITLEMENT"]] = FieldInfo(
        alias="sourceType", default=None
    )
    """The source type of the grant (PRICE, PLAN_ENTITLEMENT, ADDON_ENTITLEMENT)"""

    status: Literal["PAYMENT_PENDING", "ACTIVE", "EXPIRED", "VOIDED", "SCHEDULED"]
    """The effective status of the credit grant"""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """Timestamp of when the record was last updated"""

    voided_at: Optional[datetime] = FieldInfo(alias="voidedAt", default=None)
    """The date when the credit grant was voided"""


class CreditGrantResponse(BaseModel):
    """Response object"""

    data: Data
    """Credit grant object representing allocated credits for a customer"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import datetime
from typing_extensions import Literal, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["SubscriptionCancelParams"]


class SubscriptionCancelParams(TypedDict, total=False):
    cancellation_action: Annotated[Literal["DEFAULT", "REVOKE_ENTITLEMENTS"], PropertyInfo(alias="cancellationAction")]
    """Action on cancellation (downgrade or revoke)"""

    cancellation_time: Annotated[
        Literal["END_OF_BILLING_PERIOD", "IMMEDIATE", "SPECIFIC_DATE"], PropertyInfo(alias="cancellationTime")
    ]
    """When to cancel (immediate, period end, or date)"""

    end_date: Annotated[Union[str, datetime], PropertyInfo(alias="endDate", format="iso8601")]
    """Subscription end date"""

    prorate: bool
    """If set, enables or disables prorating of credits on subscription cancellation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["CustomerListResourcesResponse"]


class CustomerListResourcesResponse(BaseModel):
    """Resource object that belongs to a customer"""

    id: str
    """Resource slug"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Timestamp of when the record was created"""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """Timestamp of when the record was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = ["CreditGetUsageResponse", "Data", "DataCurrency", "DataSeries", "DataSeriesPoint"]


class DataCurrency(BaseModel):
    """The custom currency used for credit measurement"""

    currency_id: str = FieldInfo(alias="currencyId")
    """The currency identifier"""

    display_name: str = FieldInfo(alias="displayName")
    """The display name of the currency"""

    plural: Optional[str] = None
    """Plural unit label"""

    singular: Optional[str] = None
    """Singular unit label"""

    symbol: Optional[str] = None
    """The currency symbol"""


class DataSeriesPoint(BaseModel):
    """A single data point in the credit usage time series"""

    timestamp: datetime
    """The timestamp of the data point"""

    value: float
    """The credit usage value at this point"""


class DataSeries(BaseModel):
    """Credit usage data for a single feature"""

    feature_id: str = FieldInfo(alias="featureId")
    """The feature ID"""

    feature_name: str = FieldInfo(alias="featureName")
    """The display name of the feature"""

    points: List[DataSeriesPoint]
    """Time-series data points for this feature"""

    total_credits: float = FieldInfo(alias="totalCredits")
    """Total credits consumed by this feature in the time range"""


class Data(BaseModel):
    """Credit usage data grouped by feature with time-series points"""

    currency: Optional[DataCurrency] = None
    """The custom currency used for credit measurement"""

    series: List[DataSeries]
    """Credit usage series grouped by feature"""


class CreditGetUsageResponse(BaseModel):
    """Response object"""

    data: Data
    """Credit usage data grouped by feature with time-series points"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["EventReportParams", "Event"]


class EventReportParams(TypedDict, total=False):
    events: Required[Iterable[Event]]
    """A list of usage events to report"""


class Event(TypedDict, total=False):
    """Raw usage event"""

    customer_id: Required[Annotated[str, PropertyInfo(alias="customerId")]]
    """Customer id"""

    event_name: Required[Annotated[str, PropertyInfo(alias="eventName")]]
    """The name of the usage event"""

    idempotency_key: Required[Annotated[str, PropertyInfo(alias="idempotencyKey")]]
    """Idempotency key"""

    dimensions: Dict[str, Union[str, float, bool]]
    """Dimensions associated with the usage event"""

    resource_id: Annotated[Optional[str], PropertyInfo(alias="resourceId")]
    """Resource id"""

    timestamp: Annotated[Union[str, datetime], PropertyInfo(format="iso8601")]
    """Timestamp"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["CustomerImportParams", "Customer"]


class CustomerImportParams(TypedDict, total=False):
    customers: Required[Iterable[Customer]]
    """List of customer objects to import"""


class Customer(TypedDict, total=False):
    id: Required[str]
    """Customer slug"""

    email: Required[Optional[str]]
    """The email of the customer"""

    name: Required[Optional[str]]
    """The name of the customer"""

    metadata: Dict[str, str]
    """Additional metadata"""

    payment_method_id: Annotated[str, PropertyInfo(alias="paymentMethodId")]
    """Billing provider payment method id"""

    updated_at: Annotated[Union[str, datetime], PropertyInfo(alias="updatedAt", format="iso8601")]
    """Timestamp of when the record was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = [
    "SubscriptionPreviewResponse",
    "Data",
    "DataImmediateInvoice",
    "DataImmediateInvoiceBillingPeriodRange",
    "DataImmediateInvoiceDiscountDetails",
    "DataImmediateInvoiceDiscount",
    "DataImmediateInvoiceLine",
    "DataBillingPeriodRange",
    "DataFreeItem",
    "DataRecurringInvoice",
    "DataRecurringInvoiceBillingPeriodRange",
    "DataRecurringInvoiceDiscountDetails",
    "DataRecurringInvoiceDiscount",
    "DataRecurringInvoiceLine",
]


class DataImmediateInvoiceBillingPeriodRange(BaseModel):
    """Billing period covered"""

    end: datetime
    """Billing period end date"""

    start: datetime
    """Billing period start date"""


class DataImmediateInvoiceDiscountDetails(BaseModel):
    """Discount breakdown"""

    code: Optional[str] = None
    """Promo code used"""

    fixed_amount: Optional[float] = FieldInfo(alias="fixedAmount", default=None)
    """Fixed discount amount"""

    percentage: Optional[float] = None
    """Percentage discount"""


class DataImmediateInvoiceDiscount(BaseModel):
    """Applied discount amount"""

    amount: float
    """Discount amount"""

    currency: str
    """Currency code"""

    description: str
    """Discount description"""


class DataImmediateInvoiceLine(BaseModel):
    """Invoice line item"""

    currency: str
    """Currency code"""

    description: str
    """Line item description"""

    sub_total: float = FieldInfo(alias="subTotal")
    """Line subtotal"""

    unit_price: float = FieldInfo(alias="unitPrice")
    """Price per unit"""

    quantity: Optional[float] = None
    """Quantity"""


class DataImmediateInvoice(BaseModel):
    """Invoice due immediately"""

    sub_total: float = FieldInfo(alias="subTotal")
    """Subtotal before discounts"""

    total: float
    """Invoice total"""

    billing_period_range: Optional[DataImmediateInvoiceBillingPeriodRange] = FieldInfo(
        alias="billingPeriodRange", default=None
    )
    """Billing period covered"""

    currency: Optional[str] = None
    """Currency code"""

    discount: Optional[float] = None
    """Total discount amount"""

    discount_details: Optional[DataImmediateInvoiceDiscountDetails] = FieldInfo(alias="discountDetails", default=None)
    """Discount breakdown"""

    discounts: Optional[List[DataImmediateInvoiceDiscount]] = None
    """Applied discounts"""

    lines: Optional[List[DataImmediateInvoiceLine]] = None
    """Line items"""

    tax: Optional[float] = None
    """Tax amount"""


class DataBillingPeriodRange(BaseModel):
    """Billing period range"""

    end: Optional[datetime] = None
    """Billing period end date"""

    start: Optional[datetime] = None
    """Billing period start date"""


class DataFreeItem(BaseModel):
    """Addon configuration"""

    id: str
    """Addon ID"""

    quantity: int
    """Number of addon instances"""


class DataRecurringInvoiceBillingPeriodRange(BaseModel):
    """Billing period covered"""

    end: datetime
    """Billing period end date"""

    start: datetime
    """Billing period start date"""


class DataRecurringInvoiceDiscountDetails(BaseModel):
    """Discount breakdown"""

    code: Optional[str] = None
    """Promo code used"""

    fixed_amount: Optional[float] = FieldInfo(alias="fixedAmount", default=None)
    """Fixed discount amount"""

    percentage: Optional[float] = None
    """Percentage discount"""


class DataRecurringInvoiceDiscount(BaseModel):
    """Applied discount amount"""

    amount: float
    """Discount amount"""

    currency: str
    """Currency code"""

    description: str
    """Discount description"""


class DataRecurringInvoiceLine(BaseModel):
    """Invoice line item"""

    currency: str
    """Currency code"""

    description: str
    """Line item description"""

    sub_total: float = FieldInfo(alias="subTotal")
    """Line subtotal"""

    unit_price: float = FieldInfo(alias="unitPrice")
    """Price per unit"""

    quantity: Optional[float] = None
    """Quantity"""


class DataRecurringInvoice(BaseModel):
    """Recurring invoice preview"""

    sub_total: float = FieldInfo(alias="subTotal")
    """Subtotal before discounts"""

    total: float
    """Invoice total"""

    billing_period_range: Optional[DataRecurringInvoiceBillingPeriodRange] = FieldInfo(
        alias="billingPeriodRange", default=None
    )
    """Billing period covered"""

    currency: Optional[str] = None
    """Currency code"""

    discount: Optional[float] = None
    """Total discount amount"""

    discount_details: Optional[DataRecurringInvoiceDiscountDetails] = FieldInfo(alias="discountDetails", default=None)
    """Discount breakdown"""

    discounts: Optional[List[DataRecurringInvoiceDiscount]] = None
    """Applied discounts"""

    lines: Optional[List[DataRecurringInvoiceLine]] = None
    """Line items"""

    tax: Optional[float] = None
    """Tax amount"""


class Data(BaseModel):
    """Pricing preview with invoices"""

    immediate_invoice: DataImmediateInvoice = FieldInfo(alias="immediateInvoice")
    """Invoice due immediately"""

    billing_period_range: Optional[DataBillingPeriodRange] = FieldInfo(alias="billingPeriodRange", default=None)
    """Billing period range"""

    free_items: Optional[List[DataFreeItem]] = FieldInfo(alias="freeItems", default=None)
    """Free items included"""

    has_scheduled_updates: Optional[bool] = FieldInfo(alias="hasScheduledUpdates", default=None)
    """Whether updates are scheduled"""

    is_plan_downgrade: Optional[bool] = FieldInfo(alias="isPlanDowngrade", default=None)
    """Whether this is a downgrade"""

    recurring_invoice: Optional[DataRecurringInvoice] = FieldInfo(alias="recurringInvoice", default=None)
    """Recurring invoice preview"""


class SubscriptionPreviewResponse(BaseModel):
    """Response object"""

    data: Data
    """Pricing preview with invoices"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from pydantic import Field as FieldInfo

from ...._utils import PropertyInfo
from ...._models import BaseModel

__all__ = [
    "EntitlementCreateResponse",
    "Data",
    "DataFeature",
    "DataFeatureResetPeriodConfiguration",
    "DataFeatureResetPeriodConfigurationYearlyResetPeriodConfig",
    "DataFeatureResetPeriodConfigurationMonthlyResetPeriodConfig",
    "DataFeatureResetPeriodConfigurationWeeklyResetPeriodConfig",
    "DataCredit",
]


class DataFeatureResetPeriodConfigurationYearlyResetPeriodConfig(BaseModel):
    """Yearly reset configuration"""

    according_to: Literal["SubscriptionStart"] = FieldInfo(alias="accordingTo")
    """Reset anchor (SubscriptionStart)"""


class DataFeatureResetPeriodConfigurationMonthlyResetPeriodConfig(BaseModel):
    """Monthly reset configuration"""

    according_to: Literal["SubscriptionStart", "StartOfTheMonth"] = FieldInfo(alias="accordingTo")
    """Reset anchor (SubscriptionStart or StartOfTheMonth)"""


class DataFeatureResetPeriodConfigurationWeeklyResetPeriodConfig(BaseModel):
    """Weekly reset configuration"""

    according_to: Literal[
        "SubscriptionStart",
        "EverySunday",
        "EveryMonday",
        "EveryTuesday",
        "EveryWednesday",
        "EveryThursday",
        "EveryFriday",
        "EverySaturday",
    ] = FieldInfo(alias="accordingTo")
    """Reset anchor (SubscriptionStart or specific day)"""


DataFeatureResetPeriodConfiguration: TypeAlias = Union[
    DataFeatureResetPeriodConfigurationYearlyResetPeriodConfig,
    DataFeatureResetPeriodConfigurationMonthlyResetPeriodConfig,
    DataFeatureResetPeriodConfigurationWeeklyResetPeriodConfig,
    None,
]


class DataFeature(BaseModel):
    """Feature entitlement response"""

    id: str
    """Unique identifier of the entitlement"""

    behavior: Literal["Increment", "Override"]
    """Entitlement behavior (Increment or Override)"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Timestamp of when the record was created"""

    description: Optional[str] = None
    """Optional description of the entitlement"""

    display_name_override: Optional[str] = FieldInfo(alias="displayNameOverride", default=None)
    """Override display name for the entitlement"""

    enum_values: Optional[List[str]] = FieldInfo(alias="enumValues", default=None)
    """Allowed enum values (for feature entitlements)"""

    has_soft_limit: Optional[bool] = FieldInfo(alias="hasSoftLimit", default=None)
    """Whether the usage limit is a soft limit (for feature entitlements)"""

    has_unlimited_usage: Optional[bool] = FieldInfo(alias="hasUnlimitedUsage", default=None)
    """Whether usage is unlimited (for feature entitlements)"""

    hidden_from_widgets: List[Literal["PAYWALL", "CUSTOMER_PORTAL", "CHECKOUT"]] = FieldInfo(alias="hiddenFromWidgets")
    """Widget types where this entitlement is hidden"""

    is_custom: Optional[bool] = FieldInfo(alias="isCustom", default=None)
    """Whether this is a custom entitlement"""

    is_granted: bool = FieldInfo(alias="isGranted")
    """Whether the entitlement is granted"""

    order: Optional[float] = None
    """Display order of the entitlement"""

    reset_period: Optional[Literal["YEAR", "MONTH", "WEEK", "DAY", "HOUR"]] = FieldInfo(
        alias="resetPeriod", default=None
    )
    """Usage reset period (for feature entitlements)"""

    reset_period_configuration: Optional[DataFeatureResetPeriodConfiguration] = FieldInfo(
        alias="resetPeriodConfiguration", default=None
    )
    """Reset period configuration (for feature entitlements)"""

    type: Literal["FEATURE"]
    """Entitlement type (FEATURE or CREDIT)"""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """Timestamp of when the record was last updated"""

    usage_limit: Optional[float] = FieldInfo(alias="usageLimit", default=None)
    """Usage limit (for feature entitlements)"""


class DataCredit(BaseModel):
    """Credit entitlement response"""

    id: str
    """Unique identifier of the entitlement"""

    amount: Optional[float] = None
    """Credit amount (for credit entitlements)"""

    behavior: Literal["Increment", "Override"]
    """Entitlement behavior (Increment or Override)"""

    cadence: Optional[Literal["MONTH", "YEAR"]] = None
    """Credit grant cadence (for credit entitlements)"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """Timestamp of when the record was created"""

    description: Optional[str] = None
    """Optional description of the entitlement"""

    display_name_override: Optional[str] = FieldInfo(alias="displayNameOverride", default=None)
    """Override display name for the entitlement"""

    hidden_from_widgets: List[Literal["PAYWALL", "CUSTOMER_PORTAL", "CHECKOUT"]] = FieldInfo(alias="hiddenFromWidgets")
    """Widget types where this entitlement is hidden"""

    is_custom: Optional[bool] = FieldInfo(alias="isCustom", default=None)
    """Whether this is a custom entitlement"""

    is_granted: bool = FieldInfo(alias="isGranted")
    """Whether the entitlement is granted"""

    order: Optional[float] = None
    """Display order of the entitlement"""

    type: Literal["CREDIT"]
    """Entitlement type (FEATURE or CREDIT)"""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """Timestamp of when the record was last updated"""


Data: TypeAlias = Annotated[Union[DataFeature, DataCredit], PropertyInfo(discriminator="type")]


class EntitlementCreateResponse(BaseModel):
    """Response object"""

    data: List[Data]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["PlanPublishParams"]


class PlanPublishParams(TypedDict, total=False):
    migration_type: Required[Annotated[Literal["NEW_CUSTOMERS", "ALL_CUSTOMERS"], PropertyInfo(alias="migrationType")]]
    """The migration type of the package"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["ProductDuplicateProductParams"]


class ProductDuplicateProductParams(TypedDict, total=False):
    body_id: Required[Annotated[str, PropertyInfo(alias="id")]]
    """The unique identifier for the entity"""

    description: Optional[str]
    """Description of the product"""

    display_name: Annotated[str, PropertyInfo(alias="displayName")]
    """Display name of the product"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ....types.v1 import (
    customer_list_params,
    customer_import_params,
    customer_update_params,
    customer_provision_params,
    customer_list_resources_params,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncMyCursorIDPage, AsyncMyCursorIDPage
from .payment_method import (
    PaymentMethodResource,
    AsyncPaymentMethodResource,
    PaymentMethodResourceWithRawResponse,
    AsyncPaymentMethodResourceWithRawResponse,
    PaymentMethodResourceWithStreamingResponse,
    AsyncPaymentMethodResourceWithStreamingResponse,
)
from ...._base_client import AsyncPaginator, make_request_options
from .promotional_entitlements import (
    PromotionalEntitlementsResource,
    AsyncPromotionalEntitlementsResource,
    PromotionalEntitlementsResourceWithRawResponse,
    AsyncPromotionalEntitlementsResourceWithRawResponse,
    PromotionalEntitlementsResourceWithStreamingResponse,
    AsyncPromotionalEntitlementsResourceWithStreamingResponse,
)
from ....types.v1.customer_response import CustomerResponse
from ....types.v1.customer_list_response import CustomerListResponse
from ....types.v1.customer_import_response import CustomerImportResponse
from ....types.v1.customer_list_resources_response import CustomerListResourcesResponse

__all__ = ["CustomersResource", "AsyncCustomersResource"]


class CustomersResource(SyncAPIResource):
    @cached_property
    def payment_method(self) -> PaymentMethodResource:
        """Operations related to customers"""
        return PaymentMethodResource(self._client)

    @cached_property
    def promotional_entitlements(self) -> PromotionalEntitlementsResource:
        """Operations related to promotional entitlements"""
        return PromotionalEntitlementsResource(self._client)

    @cached_property
    def with_raw_response(self) -> CustomersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return CustomersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CustomersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return CustomersResourceWithStreamingResponse(self)

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerResponse:
        """
        Retrieves a customer by their unique identifier, including billing information
        and subscription status.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/api/v1/customers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerResponse,
        )

    def update(
        self,
        id: str,
        *,
        billing_id: Optional[str] | Omit = omit,
        coupon_id: Optional[str] | Omit = omit,
        email: Optional[str] | Omit = omit,
        integrations: Iterable[customer_update_params.Integration] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerResponse:
        """
        Updates an existing customer's properties such as name, email, and billing
        information.

        Args:
          billing_id: The unique identifier for the entity in the billing provider

          coupon_id: Customer level coupon

          email: The email of the customer

          integrations: List of integrations

          metadata: Additional metadata

          name: The name of the customer

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/api/v1/customers/{id}",
            body=maybe_transform(
                {
                    "billing_id": billing_id,
                    "coupon_id": coupon_id,
                    "email": email,
                    "integrations": integrations,
                    "metadata": metadata,
                    "name": name,
                },
                customer_update_params.CustomerUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerResponse,
        )

    def list(
        self,
        *,
        after: str | Omit = omit,
        before: str | Omit = omit,
        created_at: customer_list_params.CreatedAt | Omit = omit,
        email: str | Omit = omit,
        limit: int | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncMyCursorIDPage[CustomerListResponse]:
        """
        Retrieves a paginated list of customers in the environment.

        Args:
          after: Return items that come after this cursor

          before: Return items that come before this cursor

          created_at: Filter by creation date using range operators: gt, gte, lt, lte

          email: Filter by exact customer email address

          limit: Maximum number of items to return

          name: Filter by exact customer name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/v1/customers",
            page=SyncMyCursorIDPage[CustomerListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "created_at": created_at,
                        "email": email,
                        "limit": limit,
                        "name": name,
                    },
                    customer_list_params.CustomerListParams,
                ),
            ),
            model=CustomerListResponse,
        )

    def archive(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerResponse:
        """Archives a customer, preventing new subscriptions.

        Optionally cancels existing
        subscriptions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/v1/customers/{id}/archive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerResponse,
        )

    def import_(
        self,
        *,
        customers: Iterable[customer_import_params.Customer],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerImportResponse:
        """Imports multiple customers in bulk.

        Used for migrating customer data from
        external systems.

        Args:
          customers: List of customer objects to import

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/customers/import",
            body=maybe_transform({"customers": customers}, customer_import_params.CustomerImportParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerImportResponse,
        )

    def list_resources(
        self,
        id: str,
        *,
        after: str | Omit = omit,
        before: str | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncMyCursorIDPage[CustomerListResourcesResponse]:
        """
        Retrieves a paginated list of resources within the same customer.

        Args:
          after: Return items that come after this cursor

          before: Return items that come before this cursor

          limit: Maximum number of items to return

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/api/v1/customers/{id}/resources",
            page=SyncMyCursorIDPage[CustomerListResourcesResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                    },
                    customer_list_resources_params.CustomerListResourcesParams,
                ),
            ),
            model=CustomerListResourcesResponse,
        )

    def provision(
        self,
        *,
        id: str,
        billing_id: Optional[str] | Omit = omit,
        coupon_id: Optional[str] | Omit = omit,
        default_payment_method: Optional[customer_provision_params.DefaultPaymentMethod] | Omit = omit,
        email: Optional[str] | Omit = omit,
        integrations: Iterable[customer_provision_params.Integration] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerResponse:
        """
        Creates a new customer and optionally provisions an initial subscription in a
        single operation.

        Args:
          id: Customer slug

          billing_id: The unique identifier for the entity in the billing provider

          coupon_id: Customer level coupon

          default_payment_method: The default payment method details

          email: The email of the customer

          integrations: List of integrations

          metadata: Additional metadata

          name: The name of the customer

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/customers",
            body=maybe_transform(
                {
                    "id": id,
                    "billing_id": billing_id,
                    "coupon_id": coupon_id,
                    "default_payment_method": default_payment_method,
                    "email": email,
                    "integrations": integrations,
                    "metadata": metadata,
                    "name": name,
                },
                customer_provision_params.CustomerProvisionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerResponse,
        )

    def unarchive(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerResponse:
        """
        Restores an archived customer, allowing them to create new subscriptions again.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/v1/customers/{id}/unarchive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerResponse,
        )


class AsyncCustomersResource(AsyncAPIResource):
    @cached_property
    def payment_method(self) -> AsyncPaymentMethodResource:
        """Operations related to customers"""
        return AsyncPaymentMethodResource(self._client)

    @cached_property
    def promotional_entitlements(self) -> AsyncPromotionalEntitlementsResource:
        """Operations related to promotional entitlements"""
        return AsyncPromotionalEntitlementsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncCustomersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCustomersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCustomersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return AsyncCustomersResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerResponse:
        """
        Retrieves a customer by their unique identifier, including billing information
        and subscription status.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/api/v1/customers/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerResponse,
        )

    async def update(
        self,
        id: str,
        *,
        billing_id: Optional[str] | Omit = omit,
        coupon_id: Optional[str] | Omit = omit,
        email: Optional[str] | Omit = omit,
        integrations: Iterable[customer_update_params.Integration] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerResponse:
        """
        Updates an existing customer's properties such as name, email, and billing
        information.

        Args:
          billing_id: The unique identifier for the entity in the billing provider

          coupon_id: Customer level coupon

          email: The email of the customer

          integrations: List of integrations

          metadata: Additional metadata

          name: The name of the customer

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/api/v1/customers/{id}",
            body=await async_maybe_transform(
                {
                    "billing_id": billing_id,
                    "coupon_id": coupon_id,
                    "email": email,
                    "integrations": integrations,
                    "metadata": metadata,
                    "name": name,
                },
                customer_update_params.CustomerUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerResponse,
        )

    def list(
        self,
        *,
        after: str | Omit = omit,
        before: str | Omit = omit,
        created_at: customer_list_params.CreatedAt | Omit = omit,
        email: str | Omit = omit,
        limit: int | Omit = omit,
        name: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CustomerListResponse, AsyncMyCursorIDPage[CustomerListResponse]]:
        """
        Retrieves a paginated list of customers in the environment.

        Args:
          after: Return items that come after this cursor

          before: Return items that come before this cursor

          created_at: Filter by creation date using range operators: gt, gte, lt, lte

          email: Filter by exact customer email address

          limit: Maximum number of items to return

          name: Filter by exact customer name

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/v1/customers",
            page=AsyncMyCursorIDPage[CustomerListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "created_at": created_at,
                        "email": email,
                        "limit": limit,
                        "name": name,
                    },
                    customer_list_params.CustomerListParams,
                ),
            ),
            model=CustomerListResponse,
        )

    async def archive(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerResponse:
        """Archives a customer, preventing new subscriptions.

        Optionally cancels existing
        subscriptions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/v1/customers/{id}/archive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerResponse,
        )

    async def import_(
        self,
        *,
        customers: Iterable[customer_import_params.Customer],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerImportResponse:
        """Imports multiple customers in bulk.

        Used for migrating customer data from
        external systems.

        Args:
          customers: List of customer objects to import

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/customers/import",
            body=await async_maybe_transform({"customers": customers}, customer_import_params.CustomerImportParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerImportResponse,
        )

    def list_resources(
        self,
        id: str,
        *,
        after: str | Omit = omit,
        before: str | Omit = omit,
        limit: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CustomerListResourcesResponse, AsyncMyCursorIDPage[CustomerListResourcesResponse]]:
        """
        Retrieves a paginated list of resources within the same customer.

        Args:
          after: Return items that come after this cursor

          before: Return items that come before this cursor

          limit: Maximum number of items to return

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/api/v1/customers/{id}/resources",
            page=AsyncMyCursorIDPage[CustomerListResourcesResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                    },
                    customer_list_resources_params.CustomerListResourcesParams,
                ),
            ),
            model=CustomerListResourcesResponse,
        )

    async def provision(
        self,
        *,
        id: str,
        billing_id: Optional[str] | Omit = omit,
        coupon_id: Optional[str] | Omit = omit,
        default_payment_method: Optional[customer_provision_params.DefaultPaymentMethod] | Omit = omit,
        email: Optional[str] | Omit = omit,
        integrations: Iterable[customer_provision_params.Integration] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        name: Optional[str] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerResponse:
        """
        Creates a new customer and optionally provisions an initial subscription in a
        single operation.

        Args:
          id: Customer slug

          billing_id: The unique identifier for the entity in the billing provider

          coupon_id: Customer level coupon

          default_payment_method: The default payment method details

          email: The email of the customer

          integrations: List of integrations

          metadata: Additional metadata

          name: The name of the customer

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/customers",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "billing_id": billing_id,
                    "coupon_id": coupon_id,
                    "default_payment_method": default_payment_method,
                    "email": email,
                    "integrations": integrations,
                    "metadata": metadata,
                    "name": name,
                },
                customer_provision_params.CustomerProvisionParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerResponse,
        )

    async def unarchive(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CustomerResponse:
        """
        Restores an archived customer, allowing them to create new subscriptions again.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/v1/customers/{id}/unarchive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CustomerResponse,
        )


class CustomersResourceWithRawResponse:
    def __init__(self, customers: CustomersResource) -> None:
        self._customers = customers

        self.retrieve = to_raw_response_wrapper(
            customers.retrieve,
        )
        self.update = to_raw_response_wrapper(
            customers.update,
        )
        self.list = to_raw_response_wrapper(
            customers.list,
        )
        self.archive = to_raw_response_wrapper(
            customers.archive,
        )
        self.import_ = to_raw_response_wrapper(
            customers.import_,
        )
        self.list_resources = to_raw_response_wrapper(
            customers.list_resources,
        )
        self.provision = to_raw_response_wrapper(
            customers.provision,
        )
        self.unarchive = to_raw_response_wrapper(
            customers.unarchive,
        )

    @cached_property
    def payment_method(self) -> PaymentMethodResourceWithRawResponse:
        """Operations related to customers"""
        return PaymentMethodResourceWithRawResponse(self._customers.payment_method)

    @cached_property
    def promotional_entitlements(self) -> PromotionalEntitlementsResourceWithRawResponse:
        """Operations related to promotional entitlements"""
        return PromotionalEntitlementsResourceWithRawResponse(self._customers.promotional_entitlements)


class AsyncCustomersResourceWithRawResponse:
    def __init__(self, customers: AsyncCustomersResource) -> None:
        self._customers = customers

        self.retrieve = async_to_raw_response_wrapper(
            customers.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            customers.update,
        )
        self.list = async_to_raw_response_wrapper(
            customers.list,
        )
        self.archive = async_to_raw_response_wrapper(
            customers.archive,
        )
        self.import_ = async_to_raw_response_wrapper(
            customers.import_,
        )
        self.list_resources = async_to_raw_response_wrapper(
            customers.list_resources,
        )
        self.provision = async_to_raw_response_wrapper(
            customers.provision,
        )
        self.unarchive = async_to_raw_response_wrapper(
            customers.unarchive,
        )

    @cached_property
    def payment_method(self) -> AsyncPaymentMethodResourceWithRawResponse:
        """Operations related to customers"""
        return AsyncPaymentMethodResourceWithRawResponse(self._customers.payment_method)

    @cached_property
    def promotional_entitlements(self) -> AsyncPromotionalEntitlementsResourceWithRawResponse:
        """Operations related to promotional entitlements"""
        return AsyncPromotionalEntitlementsResourceWithRawResponse(self._customers.promotional_entitlements)


class CustomersResourceWithStreamingResponse:
    def __init__(self, customers: CustomersResource) -> None:
        self._customers = customers

        self.retrieve = to_streamed_response_wrapper(
            customers.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            customers.update,
        )
        self.list = to_streamed_response_wrapper(
            customers.list,
        )
        self.archive = to_streamed_response_wrapper(
            customers.archive,
        )
        self.import_ = to_streamed_response_wrapper(
            customers.import_,
        )
        self.list_resources = to_streamed_response_wrapper(
            customers.list_resources,
        )
        self.provision = to_streamed_response_wrapper(
            customers.provision,
        )
        self.unarchive = to_streamed_response_wrapper(
            customers.unarchive,
        )

    @cached_property
    def payment_method(self) -> PaymentMethodResourceWithStreamingResponse:
        """Operations related to customers"""
        return PaymentMethodResourceWithStreamingResponse(self._customers.payment_method)

    @cached_property
    def promotional_entitlements(self) -> PromotionalEntitlementsResourceWithStreamingResponse:
        """Operations related to promotional entitlements"""
        return PromotionalEntitlementsResourceWithStreamingResponse(self._customers.promotional_entitlements)


class AsyncCustomersResourceWithStreamingResponse:
    def __init__(self, customers: AsyncCustomersResource) -> None:
        self._customers = customers

        self.retrieve = async_to_streamed_response_wrapper(
            customers.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            customers.update,
        )
        self.list = async_to_streamed_response_wrapper(
            customers.list,
        )
        self.archive = async_to_streamed_response_wrapper(
            customers.archive,
        )
        self.import_ = async_to_streamed_response_wrapper(
            customers.import_,
        )
        self.list_resources = async_to_streamed_response_wrapper(
            customers.list_resources,
        )
        self.provision = async_to_streamed_response_wrapper(
            customers.provision,
        )
        self.unarchive = async_to_streamed_response_wrapper(
            customers.unarchive,
        )

    @cached_property
    def payment_method(self) -> AsyncPaymentMethodResourceWithStreamingResponse:
        """Operations related to customers"""
        return AsyncPaymentMethodResourceWithStreamingResponse(self._customers.payment_method)

    @cached_property
    def promotional_entitlements(self) -> AsyncPromotionalEntitlementsResourceWithStreamingResponse:
        """Operations related to promotional entitlements"""
        return AsyncPromotionalEntitlementsResourceWithStreamingResponse(self._customers.promotional_entitlements)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import TypedDict

__all__ = ["CouponUpdateCouponParams"]


class CouponUpdateCouponParams(TypedDict, total=False):
    description: Optional[str]
    """Description of the coupon"""

    metadata: Optional[Dict[str, str]]
    """Metadata associated with the entity"""

    name: str
    """Name of the coupon"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = ["SubscriptionTransferParams"]


class SubscriptionTransferParams(TypedDict, total=False):
    destination_resource_id: Required[Annotated[str, PropertyInfo(alias="destinationResourceId")]]
    """Resource ID to transfer the subscription to"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["InvoiceMarkAsPaidResponse", "Data"]


class Data(BaseModel):
    """Result of marking a subscription invoice as paid."""

    id: str
    """The subscription ID whose invoice was marked as paid"""


class InvoiceMarkAsPaidResponse(BaseModel):
    """Response object"""

    data: Data
    """Result of marking a subscription invoice as paid."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ....types.v1 import (
    addon_list_params,
    addon_create_params,
    addon_update_params,
    addon_publish_params,
    addon_set_pricing_params,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .entitlements import (
    EntitlementsResource,
    AsyncEntitlementsResource,
    EntitlementsResourceWithRawResponse,
    AsyncEntitlementsResourceWithRawResponse,
    EntitlementsResourceWithStreamingResponse,
    AsyncEntitlementsResourceWithStreamingResponse,
)
from ....pagination import SyncMyCursorIDPage, AsyncMyCursorIDPage
from ...._base_client import AsyncPaginator, make_request_options
from ....types.v1.addon import Addon
from ....types.v1.addon_list_response import AddonListResponse
from ....types.v1.addon_publish_response import AddonPublishResponse
from ....types.v1.addon_remove_draft_response import AddonRemoveDraftResponse
from ....types.v1.set_package_pricing_response import SetPackagePricingResponse

__all__ = ["AddonsResource", "AsyncAddonsResource"]


class AddonsResource(SyncAPIResource):
    """Operations related to addons"""

    @cached_property
    def entitlements(self) -> EntitlementsResource:
        return EntitlementsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AddonsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return AddonsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AddonsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return AddonsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        id: str,
        display_name: str,
        product_id: str,
        billing_id: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        max_quantity: Optional[int] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        pricing_type: Optional[Literal["FREE", "PAID", "CUSTOM"]] | Omit = omit,
        status: Literal["DRAFT", "PUBLISHED", "ARCHIVED"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Addon:
        """
        Creates a new addon in draft status, associated with a specific product.

        Args:
          id: The unique identifier for the entity

          display_name: The display name of the package

          product_id: The product id of the package

          billing_id: The unique identifier for the entity in the billing provider

          description: The description of the package

          max_quantity: The maximum quantity of this addon that can be added to a subscription

          metadata: Metadata associated with the entity

          pricing_type: The pricing type of the package

          status: The status of the package

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/api/v1/addons",
            body=maybe_transform(
                {
                    "id": id,
                    "display_name": display_name,
                    "product_id": product_id,
                    "billing_id": billing_id,
                    "description": description,
                    "max_quantity": max_quantity,
                    "metadata": metadata,
                    "pricing_type": pricing_type,
                    "status": status,
                },
                addon_create_params.AddonCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Addon,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Addon:
        """
        Retrieves an addon by its unique identifier, including entitlements and pricing
        details.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/api/v1/addons/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Addon,
        )

    def update(
        self,
        id: str,
        *,
        billing_id: Optional[str] | Omit = omit,
        dependencies: Optional[SequenceNotStr[str]] | Omit = omit,
        description: Optional[str] | Omit = omit,
        display_name: str | Omit = omit,
        max_quantity: Optional[int] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        status: Literal["DRAFT", "PUBLISHED", "ARCHIVED"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Addon:
        """
        Updates an existing addon's properties such as display name, description, and
        metadata.

        Args:
          billing_id: The unique identifier for the entity in the billing provider

          dependencies: List of addons the addon is dependant on

          description: The description of the package

          display_name: The display name of the package

          max_quantity: The maximum quantity of this addon that can be added to a subscription

          metadata: Metadata associated with the entity

          status: The status of the package

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/api/v1/addons/{id}",
            body=maybe_transform(
                {
                    "billing_id": billing_id,
                    "dependencies": dependencies,
                    "description": description,
                    "display_name": display_name,
                    "max_quantity": max_quantity,
                    "metadata": metadata,
                    "status": status,
                },
                addon_update_params.AddonUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Addon,
        )

    def list(
        self,
        *,
        after: str | Omit = omit,
        before: str | Omit = omit,
        created_at: addon_list_params.CreatedAt | Omit = omit,
        limit: int | Omit = omit,
        product_id: str | Omit = omit,
        status: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncMyCursorIDPage[AddonListResponse]:
        """
        Retrieves a paginated list of addons in the environment.

        Args:
          after: Return items that come after this cursor

          before: Return items that come before this cursor

          created_at: Filter by creation date using range operators: gt, gte, lt, lte

          limit: Maximum number of items to return

          product_id: Filter by product ID

          status: Filter by status. Supports comma-separated values for multiple statuses

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/v1/addons",
            page=SyncMyCursorIDPage[AddonListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "created_at": created_at,
                        "limit": limit,
                        "product_id": product_id,
                        "status": status,
                    },
                    addon_list_params.AddonListParams,
                ),
            ),
            model=AddonListResponse,
        )

    def archive(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Addon:
        """
        Archives an addon, preventing it from being used in new subscriptions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/v1/addons/{id}/archive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Addon,
        )

    def create_draft(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Addon:
        """
        Creates a draft version of an existing addon for modification before publishing.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/v1/addons/{id}/draft",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Addon,
        )

    def publish(
        self,
        id: str,
        *,
        migration_type: Literal["NEW_CUSTOMERS", "ALL_CUSTOMERS"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddonPublishResponse:
        """
        Publishes a draft addon, making it available for use in subscriptions.

        Args:
          migration_type: The migration type of the package

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/api/v1/addons/{id}/publish",
            body=maybe_transform({"migration_type": migration_type}, addon_publish_params.AddonPublishParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AddonPublishResponse,
        )

    def remove_draft(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddonRemoveDraftResponse:
        """
        Removes a draft version of an addon.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/api/v1/addons/{id}/draft",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AddonRemoveDraftResponse,
        )

    def set_pricing(
        self,
        id: str,
        *,
        pricing_type: Literal["FREE", "PAID", "CUSTOM"],
        billing_id: str | Omit = omit,
        minimum_spend: Optional[Iterable[addon_set_pricing_params.MinimumSpend]] | Omit = omit,
        overage_billing_period: Literal["ON_SUBSCRIPTION_RENEWAL", "MONTHLY"] | Omit = omit,
        overage_pricing_models: Iterable[addon_set_pricing_params.OveragePricingModel] | Omit = omit,
        pricing_models: Iterable[addon_set_pricing_params.PricingModel] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SetPackagePricingResponse:
        """
        Sets the pricing configuration for an addon.

        Args:
          pricing_type: The pricing type (FREE, PAID, or CUSTOM)

          billing_id: Deprecated: billing integration ID

          minimum_spend: Minimum spend configuration per billing period

          overage_billing_period: When overage charges are billed

          overage_pricing_models: Array of overage pricing model configurations

          pricing_models: Array of pricing model configurations

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._put(
            f"/api/v1/addons/{id}/charges",
            body=maybe_transform(
                {
                    "pricing_type": pricing_type,
                    "billing_id": billing_id,
                    "minimum_spend": minimum_spend,
                    "overage_billing_period": overage_billing_period,
                    "overage_pricing_models": overage_pricing_models,
                    "pricing_models": pricing_models,
                },
                addon_set_pricing_params.AddonSetPricingParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SetPackagePricingResponse,
        )


class AsyncAddonsResource(AsyncAPIResource):
    """Operations related to addons"""

    @cached_property
    def entitlements(self) -> AsyncEntitlementsResource:
        return AsyncEntitlementsResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncAddonsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/stiggio/stigg-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAddonsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAddonsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/stiggio/stigg-python#with_streaming_response
        """
        return AsyncAddonsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        id: str,
        display_name: str,
        product_id: str,
        billing_id: Optional[str] | Omit = omit,
        description: Optional[str] | Omit = omit,
        max_quantity: Optional[int] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        pricing_type: Optional[Literal["FREE", "PAID", "CUSTOM"]] | Omit = omit,
        status: Literal["DRAFT", "PUBLISHED", "ARCHIVED"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Addon:
        """
        Creates a new addon in draft status, associated with a specific product.

        Args:
          id: The unique identifier for the entity

          display_name: The display name of the package

          product_id: The product id of the package

          billing_id: The unique identifier for the entity in the billing provider

          description: The description of the package

          max_quantity: The maximum quantity of this addon that can be added to a subscription

          metadata: Metadata associated with the entity

          pricing_type: The pricing type of the package

          status: The status of the package

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/api/v1/addons",
            body=await async_maybe_transform(
                {
                    "id": id,
                    "display_name": display_name,
                    "product_id": product_id,
                    "billing_id": billing_id,
                    "description": description,
                    "max_quantity": max_quantity,
                    "metadata": metadata,
                    "pricing_type": pricing_type,
                    "status": status,
                },
                addon_create_params.AddonCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Addon,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Addon:
        """
        Retrieves an addon by its unique identifier, including entitlements and pricing
        details.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/api/v1/addons/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Addon,
        )

    async def update(
        self,
        id: str,
        *,
        billing_id: Optional[str] | Omit = omit,
        dependencies: Optional[SequenceNotStr[str]] | Omit = omit,
        description: Optional[str] | Omit = omit,
        display_name: str | Omit = omit,
        max_quantity: Optional[int] | Omit = omit,
        metadata: Dict[str, str] | Omit = omit,
        status: Literal["DRAFT", "PUBLISHED", "ARCHIVED"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Addon:
        """
        Updates an existing addon's properties such as display name, description, and
        metadata.

        Args:
          billing_id: The unique identifier for the entity in the billing provider

          dependencies: List of addons the addon is dependant on

          description: The description of the package

          display_name: The display name of the package

          max_quantity: The maximum quantity of this addon that can be added to a subscription

          metadata: Metadata associated with the entity

          status: The status of the package

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/api/v1/addons/{id}",
            body=await async_maybe_transform(
                {
                    "billing_id": billing_id,
                    "dependencies": dependencies,
                    "description": description,
                    "display_name": display_name,
                    "max_quantity": max_quantity,
                    "metadata": metadata,
                    "status": status,
                },
                addon_update_params.AddonUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Addon,
        )

    def list(
        self,
        *,
        after: str | Omit = omit,
        before: str | Omit = omit,
        created_at: addon_list_params.CreatedAt | Omit = omit,
        limit: int | Omit = omit,
        product_id: str | Omit = omit,
        status: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AddonListResponse, AsyncMyCursorIDPage[AddonListResponse]]:
        """
        Retrieves a paginated list of addons in the environment.

        Args:
          after: Return items that come after this cursor

          before: Return items that come before this cursor

          created_at: Filter by creation date using range operators: gt, gte, lt, lte

          limit: Maximum number of items to return

          product_id: Filter by product ID

          status: Filter by status. Supports comma-separated values for multiple statuses

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/api/v1/addons",
            page=AsyncMyCursorIDPage[AddonListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "created_at": created_at,
                        "limit": limit,
                        "product_id": product_id,
                        "status": status,
                    },
                    addon_list_params.AddonListParams,
                ),
            ),
            model=AddonListResponse,
        )

    async def archive(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Addon:
        """
        Archives an addon, preventing it from being used in new subscriptions.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/v1/addons/{id}/archive",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Addon,
        )

    async def create_draft(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Addon:
        """
        Creates a draft version of an existing addon for modification before publishing.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/v1/addons/{id}/draft",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Addon,
        )

    async def publish(
        self,
        id: str,
        *,
        migration_type: Literal["NEW_CUSTOMERS", "ALL_CUSTOMERS"],
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddonPublishResponse:
        """
        Publishes a draft addon, making it available for use in subscriptions.

        Args:
          migration_type: The migration type of the package

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/api/v1/addons/{id}/publish",
            body=await async_maybe_transform(
                {"migration_type": migration_type}, addon_publish_params.AddonPublishParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AddonPublishResponse,
        )

    async def remove_draft(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AddonRemoveDraftResponse:
        """
        Removes a draft version of an addon.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/api/v1/addons/{id}/draft",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AddonRemoveDraftResponse,
        )

    async def set_pricing(
        self,
        id: str,
        *,
        pricing_type: Literal["FREE", "PAID", "CUSTOM"],
        billing_id: str | Omit = omit,
        minimum_spend: Optional[Iterable[addon_set_pricing_params.MinimumSpend]] | Omit = omit,
        overage_billing_period: Literal["ON_SUBSCRIPTION_RENEWAL", "MONTHLY"] | Omit = omit,
        overage_pricing_models: Iterable[addon_set_pricing_params.OveragePricingModel] | Omit = omit,
        pricing_models: Iterable[addon_set_pricing_params.PricingModel] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SetPackagePricingResponse:
        """
        Sets the pricing configuration for an addon.

        Args:
          pricing_type: The pricing type (FREE, PAID, or CUSTOM)

          billing_id: Deprecated: billing integration ID

          minimum_spend: Minimum spend configuration per billing period

          overage_billing_period: When overage charges are billed

          overage_pricing_models: Array of overage pricing model configurations

          pricing_models: Array of pricing model configurations

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._put(
            f"/api/v1/addons/{id}/charges",
            body=await async_maybe_transform(
                {
                    "pricing_type": pricing_type,
                    "billing_id": billing_id,
                    "minimum_spend": minimum_spend,
                    "overage_billing_period": overage_billing_period,
                    "overage_pricing_models": overage_pricing_models,
                    "pricing_models": pricing_models,
                },
                addon_set_pricing_params.AddonSetPricingParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=SetPackagePricingResponse,
        )


class AddonsResourceWithRawResponse:
    def __init__(self, addons: AddonsResource) -> None:
        self._addons = addons

        self.create = to_raw_response_wrapper(
            addons.create,
        )
        self.retrieve = to_raw_response_wrapper(
            addons.retrieve,
        )
        self.update = to_raw_response_wrapper(
            addons.update,
        )
        self.list = to_raw_response_wrapper(
            addons.list,
        )
        self.archive = to_raw_response_wrapper(
            addons.archive,
        )
        self.create_draft = to_raw_response_wrapper(
            addons.create_draft,
        )
        self.publish = to_raw_response_wrapper(
            addons.publish,
        )
        self.remove_draft = to_raw_response_wrapper(
            addons.remove_draft,
        )
        self.set_pricing = to_raw_response_wrapper(
            addons.set_pricing,
        )

    @cached_property
    def entitlements(self) -> EntitlementsResourceWithRawResponse:
        return EntitlementsResourceWithRawResponse(self._addons.entitlements)


class AsyncAddonsResourceWithRawResponse:
    def __init__(self, addons: AsyncAddonsResource) -> None:
        self._addons = addons

        self.create = async_to_raw_response_wrapper(
            addons.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            addons.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            addons.update,
        )
        self.list = async_to_raw_response_wrapper(
            addons.list,
        )
        self.archive = async_to_raw_response_wrapper(
            addons.archive,
        )
        self.create_draft = async_to_raw_response_wrapper(
            addons.create_draft,
        )
        self.publish = async_to_raw_response_wrapper(
            addons.publish,
        )
        self.remove_draft = async_to_raw_response_wrapper(
            addons.remove_draft,
        )
        self.set_pricing = async_to_raw_response_wrapper(
            addons.set_pricing,
        )

    @cached_property
    def entitlements(self) -> AsyncEntitlementsResourceWithRawResponse:
        return AsyncEntitlementsResourceWithRawResponse(self._addons.entitlements)


class AddonsResourceWithStreamingResponse:
    def __init__(self, addons: AddonsResource) -> None:
        self._addons = addons

        self.create = to_streamed_response_wrapper(
            addons.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            addons.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            addons.update,
        )
        self.list = to_streamed_response_wrapper(
            addons.list,
        )
        self.archive = to_streamed_response_wrapper(
            addons.archive,
        )
        self.create_draft = to_streamed_response_wrapper(
            addons.create_draft,
        )
        self.publish = to_streamed_response_wrapper(
            addons.publish,
        )
        self.remove_draft = to_streamed_response_wrapper(
            addons.remove_draft,
        )
        self.set_pricing = to_streamed_response_wrapper(
            addons.set_pricing,
        )

    @cached_property
    def entitlements(self) -> EntitlementsResourceWithStreamingResponse:
        return EntitlementsResourceWithStreamingResponse(self._addons.entitlements)


class AsyncAddonsResourceWithStreamingResponse:
    def __init__(self, addons: AsyncAddonsResource) -> None:
        self._addons = addons

        self.create = async_to_streamed_response_wrapper(
            addons.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            addons.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            addons.update,
        )
        self.list = async_to_streamed_response_wrapper(
            addons.list,
        )
        self.archive = async_to_streamed_response_wrapper(
            addons.archive,
        )
        self.create_draft = async_to_streamed_response_wrapper(
            addons.create_draft,
        )
        self.publish = async_to_streamed_response_wrapper(
            addons.publish,
        )
        self.remove_draft = async_to_streamed_response_wrapper(
            addons.remove_draft,
        )
        self.set_pricing = async_to_streamed_response_wrapper(
            addons.set_pricing,
        )

    @cached_property
    def entitlements(self) -> AsyncEntitlementsResourceWithStreamingResponse:
        return AsyncEntitlementsResourceWithStreamingResponse(self._addons.entitlements)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["SetPackagePricingResponse", "Data"]


class Data(BaseModel):
    """Result of setting package pricing."""

    id: str
    """The package identifier (refId)"""

    pricing_type: Literal["FREE", "PAID", "CUSTOM"] = FieldInfo(alias="pricingType")
    """The pricing type that was set"""


class SetPackagePricingResponse(BaseModel):
    """Response object"""

    data: Data
    """Result of setting package pricing."""
